/*
 * defs.h - Misc. definitions.
 *
 * Copyright (c) 2003-2004 Marcus R. Brown <mrbrown@0xd6.org>
 *
 * Licensed under the AFL v2.0. See the file LICENSE included with this
 * distribution for licensing terms.
 */
#ifndef _DEFS_H_
#define _DEFS_H_

#include "types.h"

#ifndef NULL
#define NULL	((void *)0)
#endif

#define ALIGN(x, align)	(((x)+((align)-1))&~((align)-1))

#define UNCACHED(x)	((__typeof__(x))((u32)(x) | 0x20000000))

#define PHYSADDR(a)	(((unsigned int)(a)) & 0x1fffffff)

#define KSEG1		0xa0000000
#define KSEG1ADDR(a)	((__typeof__(a))(((u32)(a) & 0x1fffffff) | KSEG1))

#define FUNC_ALIAS(name, symbol) \
__asm__ (".globl\t" name "\n\t"	name ": .alias\t" symbol)

#define OS_SYSCALL(name, number, type, argdef...) \
type name( ## argdef)	\
{			\
	__asm__ volatile ("li\t$3, %0\n\tsyscall\n\t" : : "i" (number)); \
}

static inline u8 _lb(u32 addr) { return *(volatile u8 *)addr; }
static inline u16 _lh(u32 addr) { return *(volatile u16 *)addr; }
static inline u32 _lw(u32 addr) { return *(volatile u32 *)addr; }
static inline u64 _ld(u32 addr) { return *(volatile u64 *)addr; }
static inline u128 _lq(u32 addr) { return *(volatile u128 *)addr; }

static inline void _sb(u8 val, u32 addr) { *(volatile u8 *)addr = val; }
static inline void _sh(u16 val, u32 addr) { *(volatile u16 *)addr = val; }
static inline void _sw(u32 val, u32 addr) { *(volatile u32 *)addr = val; }
static inline void _sd(u64 val, u32 addr) { *(volatile u64 *)addr = val; }
static inline void _sq(u128 val, u32 addr) { *(volatile u128 *)addr = val; }

#define memcpy slow_memcpy
static inline void * slow_memcpy(void *dest, const void *src, int size)
{
	u8 *d = (u8 *)dest, *s = (u8 *)src;

	while (size--)
		*d++ = *s++;

	return dest;
}

#define memset slow_memset
static inline void * slow_memset(void *dest, int c, int size)
{
	u8 *d = (u8 *)dest;

	while (size-- != 0)
		*d++ = (u8)c;

	return dest;
}

#endif /* _DEFS_H_ */
